// Copyright 2022 ESRI
//
// All rights reserved under the copyright laws of the United States
// and applicable international laws, treaties, and conventions.
//
// You may freely redistribute and use this sample code, with or
// without modification, provided you include the original copyright
// notice and use restrictions.
//
// See the Sample code usage restrictions document for further information.
//

#include "Display_a_map.h"
#include "Map.h"
#include "MapTypes.h"
#include "MapQuickView.h"
#include "Point.h"
#include "Viewpoint.h"
#include "SpatialReference.h"
#include <QFuture>

#include "LayerListModel.h"
#include "ServiceFeatureTable.h"
#include "FeatureLayer.h"

using namespace Esri::ArcGISRuntime;

Display_a_map::Display_a_map(QObject* parent /* = nullptr */):
  QObject(parent),
  m_map(new Map(BasemapStyle::ArcGISTopographic, this))
{
}

Display_a_map::~Display_a_map()
{
}

MapQuickView* Display_a_map::mapView() const
{
  return m_mapView;
}

void Display_a_map::setupViewpoint()
{
  const Point center(-118.80543, 34.02700, SpatialReference::wgs84());
  const Viewpoint viewpoint(center, 100000.0);
  m_mapView->setViewpointAsync(viewpoint);

  const QUrl polygonFeatureUrl("https://services3.arcgis.com/GVgbJbqm8hXASVYi/ArcGIS/rest/services/Parks_and_Open_Space/FeatureServer/0");
  ServiceFeatureTable* polygon_feature_table = new ServiceFeatureTable(polygonFeatureUrl, this);

  FeatureLayer* polygon_feature_layer = new FeatureLayer(polygon_feature_table, this);
  m_map->operationalLayers()->append(polygon_feature_layer);

  const QUrl lineFeatureUrl("https://services3.arcgis.com/GVgbJbqm8hXASVYi/ArcGIS/rest/services/Trails/FeatureServer/0");
  ServiceFeatureTable* line_feature_table = new ServiceFeatureTable(lineFeatureUrl, this);

  FeatureLayer* line_feature_layer = new FeatureLayer(line_feature_table, this);
  m_map->operationalLayers()->append(line_feature_layer);

  const QUrl pointFeatureUrl("https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/Trailheads/FeatureServer/0");
  ServiceFeatureTable* point_feature_table= new ServiceFeatureTable(pointFeatureUrl, this);

  FeatureLayer* point_feature_layer = new FeatureLayer(point_feature_table, this);
  m_map->operationalLayers()->append(point_feature_layer);

}
// Set the view (created in QML)
void Display_a_map::setMapView(MapQuickView* mapView)
{
  if (!mapView || mapView == m_mapView)
  {
    return;
  }

  m_mapView = mapView;
  m_mapView->setMap(m_map);

  setupViewpoint();

  emit mapViewChanged();
}

